#ifndef _DEBUG
#define _DEBUG
#endif

#include <unistd.h>
#include <stdio.h>
#include <iostream>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/time.h> 
#include <netdb.h>
#include <string>
#include <errno.h>
#include <signal.h> 
#include <stdlib.h>
#include <fcntl.h>

#define MAXLINE 4096
#define RECONN_DELAY 3
#define LOCAL_ENDPOINT 9001
#define PEER_ENDPOINT  9002

typedef int SOCKDESCR;	//socket descriptor
typedef struct sockaddr_in SA_IN;  //alias for internet socket address
typedef struct sockaddr SA;
typedef void Sigfunc(int);

//Socket returns a valid socket descriptor if succeded or -1 if fails
SOCKDESCR Socket(int family, int type, int protocol);

//Connect returns a valid socket descriptor if succeded or -1 if fails
bool Connect(SOCKDESCR sockfd, const SA *psrvaddr, socklen_t srvaddrlen);

bool ConnectWithTimeout(SOCKDESCR sockfd, const SA *psrvaddr, socklen_t srvaddrlen, int timeout);

//Reconnect ritorna -1 e socket chiusa se non riesce a connettersi
SOCKDESCR Reconnect(SOCKDESCR sockfd,const SA* psrvaddr, socklen_t srvaddrlen, int resumetime);

//Bind returns true if succeded, false if error occurs
bool Bind(SOCKDESCR sockfd, const SA* psockaddr, socklen_t sockaddrlen);

//Rebind cerca di appioppare una ephemeral port in [lowerbound,upperbound]
//notare che il 2 parametro NON e' piu' const
int Rebind(SOCKDESCR sockfd, SA_IN *psockaddr, socklen_t sockaddrlen, int lowerbound, int upperbound);

//Listen return true in succeded, false if error occurs
bool Listen(SOCKDESCR sockfd, int backlog);

//Accept returns a valid socket descriptor if succeded, -1 if fails
SOCKDESCR Accept(SOCKDESCR sockfd, SA* pcliaddr, socklen_t* pcliaddrlen);

bool Close(SOCKDESCR sockfd);

//send and recv rimasticate un pelo
int Send(SOCKDESCR sock, const void *msg, int msglen, int sendflag, int timeout);
int Recv(SOCKDESCR sock, void *buffer, int bufflen, int recvflags, int timelimit);

string GetIPAddr(SOCKDESCR sockfd, int endpointflag);
int    GetIPPort(SOCKDESCR sockfd, int endpointflag);	

//write a line into fd with error handling
ssize_t Readline(int fd, void* ptr, size_t maxlen);
//write a chunk into fd wirh error handling
void Writen(int fd, void* ptr, size_t nbytes);
//read a chunk from fd with error handling
ssize_t Readn(int fd, void* ptr, size_t nbytes);
