#include "mysocketheader.h"

//-----------------------------------------
SOCKDESCR Socket(int family, int type, int protocol)
{
   //open a TCP socket and catch errors
	SOCKDESCR retvalue = -1;
	if( (retvalue = socket(family,type,protocol)) > 0)
		return retvalue;
	else
	{
		int ler = errno;
		cerr<<"Cannot open socket"<<endl;
		errno = ler;
		perror("Reason: ");
		return retvalue;
	}
}
//------------------------------            
bool Connect(SOCKDESCR sockfd, const SA *paddr, socklen_t addrlen)                  
{
	//ask for a connection
	if(connect(sockfd,paddr,addrlen) < 0)
	{
		int ler = errno;
		cerr<<"Cant Connect"<<endl;
		errno =ler;
		perror("Reason: ");
		return false;
	}
	else
		return true;
}
//-----------------------------------------------------------------------
static void connect_alarm(int second)
{
	return;
}
bool ConnectWithTimeout(SOCKDESCR sockfd,const SA *paddr,socklen_t addrlen,int timeout)
{
	Sigfunc *sigfunc = signal(SIGALRM,connect_alarm);
	
	int remsec = 0;
	if( (remsec = alarm(timeout)) != 0)
		cerr<<"warning: overwriting previous alarm signal"<<endl;
	
	int retval = 0;
	if( (retval = connect(sockfd,paddr,addrlen)) < 0){
		if(errno == EINTR) errno = ETIMEDOUT;
		alarm(remsec);  //ristore del precedente alarm //traslato di timeout
		signal(SIGALRM,sigfunc);
		return false;
	}else{
		alarm(remsec);
		signal(SIGALRM,sigfunc);
		return true;
	}
}		
//------------------------------
//restituisce -1 nel caso non si riesca a stabilire una connessione
//chiude la socket in caso di errore;
//in caso di successo restituisce il socket descriptor;
SOCKDESCR Reconnect(SOCKDESCR sockfd, const SA *paddr, socklen_t addrlen, int maxresume)
{	
	int resumetime = 0;
	bool goon = true;
	while(goon){
		if(!Connect(sockfd,paddr,addrlen))
			if(errno == ECONNREFUSED){
				cout<<"connection refused. retrying..."<<endl;
				Close(sockfd);
				sleep(RECONN_DELAY); //riprova dopo un secondo
				if( (sockfd = Socket(AF_INET,SOCK_STREAM,0)) == -1){
					perror("Client CANNOT start up");
					exit(-1);
				}
				resumetime++;
				if(resumetime > maxresume) goon = false;
			}else if(errno == ETIMEDOUT) goon = false;
				else{
					cerr<<"uncatched error. "<<endl;
					perror("");
					return -1;
				}
	 	else{
			return sockfd;
		}
	}//fine while
	Close(sockfd);
	return -1;
}	
//-------------------------------                  
bool Bind(SOCKDESCR sockfd, const SA* psockaddr, socklen_t sockaddrlen)
{
	//try to bind socket to port
	if(bind(sockfd, psockaddr, sockaddrlen) < 0)	{
#ifdef _DEBUG	
		int ler =errno;
		cerr<<" Cannot bind local address"<<endl;
		errno=ler;
		perror("Reason: ");
#endif	
		return false;
	}
	else
		return true;
}
//----------------------------------------------
int Rebind(SOCKDESCR sockfd, SA_IN *psockaddr, socklen_t sockaddrlen, int lowerbound, int upperbound)
{
	//cerca di fire il bind con una porta entro il range [lowerbound,upperbound]
	while((lowerbound <= upperbound)){
		psockaddr->sin_port = htons(lowerbound);
		if(!Bind(sockfd,(SA *)psockaddr,sockaddrlen)) lowerbound++;
		else return lowerbound;
	}
	return -1;
}			
//----------------------------
bool Listen(SOCKDESCR sockfd, int backlog)
{
	//try to put up a passive socket
	if(listen(sockfd,backlog) < 0)
	{
		int ler = errno;
		cerr<<"CANNOT losten up socket"<<endl;
		errno = ler;
		perror("Reason: ");
		return false;
	}
	else
		return true;
}
//-------------------------------                  
SOCKDESCR Accept(SOCKDESCR sockfd, SA* paddr, socklen_t* paddrlen)
{
	//try to Accept a connection
	SOCKDESCR retvalue = -1;
	if( (retvalue = accept(sockfd, paddr, paddrlen)) < 0)
	{
		int ler = errno;
		cerr<<"CANNOT accept connectio"<<endl;
		errno = ler;
		perror("Reason: ");
		return false;
	}
	else
		return retvalue;
}
//---------------------------------------- 
bool Close(SOCKDESCR sockfd)
{
	//try to close a socket
	if(close(sockfd) < 0)
	{
		int ler = errno;
		cerr<<"CANNOT Close socket"<<endl;
	 	errno = ler;
		perror("Reason ");
		return false;
	}
	else 
		return true;
}
//-----------------------------------------------------
string GetIPAddr(SOCKDESCR sockfd,int endpointflag)
{
	char temp[16];
	memset(temp,0,sizeof(temp));
	
	struct sockaddr tempaddr;
	socklen_t tempaddrlen = sizeof(tempaddr);
	
	switch(endpointflag){
		case LOCAL_ENDPOINT:
			getsockname(sockfd,&tempaddr,&tempaddrlen);
		break;
		case PEER_ENDPOINT:
			getpeername(sockfd,&tempaddr,&tempaddrlen);
		break;
		default:
			cerr<<"BAD GetIPAddr flag"<<endl;
	}
	SA_IN *paddr_in;
	paddr_in = (SA_IN *)&tempaddr;
		
	if(inet_ntop(AF_INET,&(paddr_in->sin_addr),temp,sizeof(temp)) == NULL){
		cerr<<"GetIPAddr inet_ntop error"<<endl;
		return "";
	}
#ifdef _DEBUG
cout<<"GetIPAddr: "<<string(temp)<<endl;
#endif

	return (string)temp;
}
///--------------------------------------------------------
int GetIPPort(SOCKDESCR sockfd, int endpointflag)
{
	struct sockaddr tempaddr;
	
	socklen_t tempaddrlen = sizeof(tempaddr);
	
	switch(endpointflag){
		case LOCAL_ENDPOINT:
			getsockname(sockfd,&tempaddr,&tempaddrlen);
		break;
		case PEER_ENDPOINT:
			getpeername(sockfd,&tempaddr,&tempaddrlen);
		break;
		default:
			cerr<<"GetIPAddr BAD flag"<<endl;
	}
	SA_IN *paddr;
	paddr = (SA_IN *)&tempaddr;
	
	return (ntohs(paddr->sin_port));
}
//----------------------------------------------------------
//questa e' una send con timeout deciso dall'utente
//il timeout e' espressoo in secondi
//returna
//i byte spediti
//-1 se timeout
//-2 se unrecover error
int Send(SOCKDESCR sock, const void *msg, int msglen, int sendflag, int timeout)
{
	int bytesent = 0;
	bytesent = send(sock,msg,msglen,sendflag);
	if(bytesent == -1) return -2;
	else return bytesent;
}	
//-------------------------------------------------------------
//recv con timeout
//-1 timeout scaduto
//-2 altri errori
//byteread se abbiamo letto

int Recv(SOCKDESCR sock, void *buffer, int bufflen, int recvflags, int timelimit)
{
	//se recvflag = 0 =>default recv flags
	//se recvflag =-1 =>default "Recv" flag =>MSG_WAITALL
	//se recvflag != 0 or -1 set new flags
	
	if(recvflags == -1) recvflags = MSG_WAITALL;
	
	/***********************************************************
	
	//mettiamo su il timeout con la select
	struct timeval timeout;
	timeout.tv_sec  = timelimit;
	timeout.tv_usec = 0;
	
	fd_set readready;
	FD_ZERO(&readready);
	FD_SET(sock,&readready);
	int maxdescr = sock+1;
	bool done = false;
	//NOTA si presume che l'OS ospite sia conforme con POSIX1.g
	//la select viene automaticamente ripresa dopo l'iterruzione
	while(!done){
		int rc = select(maxdescr, &readready, (fd_set *)0, (fd_set *)0, &timeout);
		if(rc < 0){
			if(errno != EINTR){
				cerr<<"something happend in recv socket"<<endl;
				perror(" ");
				return -1;
			}
		}else if(rc == 0)	return -2;
				else done = true;
	}
	
	//ok ricevuto qualcosa prima del timeout
	//chiamo la recv
************************************************************/
	int byterecv = recv(sock,buffer,bufflen,recvflags);
	if(byterecv < 0){
		cerr<<"Recv unrecoverable error"<<endl;
		perror(" ");
		return -2;
	}
	else return byterecv;
}
//------------------------------------------------------			
	